==> GPGee-1.2.0-source.zip: gpgee/gpgme/w32-util.c <==
/* w32-util.c - Utility functions for the W32 API
 *	Copyright (C) 1999 Free Software Foundation, Inc
 *	Copyright (C) 2001 Werner Koch (dd9jn)
 *	Copyright (C) 2002, 2005 Timo Schulz
 *
 * This file is part of MyGPGME.
 *
 * MyGPGME is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * MyGPGME is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>
#include <errno.h>
#include <sys/types.h>
#include <windows.h>
#include <process.h>
#include <io.h>

#include "util.h"

extern char *_gpgme_gpg_program;
extern char *_gpgme_gpg_optfile;
extern char *_gpgme_gpg_pubring;
extern char *_gpgme_gpg_secring;


/****************
 * Return a string from the Win32 Registry or NULL in case of
 * error.  Caller must release the return value.   A NULL for root
 * is an alias fro HKEY_CURRENT_USER
 */
static char *
read_w32_registry_string ( const char *root,
                           const char *dir, const char *name )
{
    HKEY root_key, key_handle;
    DWORD n1, nbytes;
    char *result = NULL;
    
    if( !root )
        root_key = HKEY_CURRENT_USER;
    else if( !strcmp( root, "HKEY_CLASSES_ROOT" ) )
        root_key = HKEY_CLASSES_ROOT;
    else if( !strcmp( root, "HKEY_CURRENT_USER" ) )
        root_key = HKEY_CURRENT_USER;
    else if( !strcmp( root, "HKEY_LOCAL_MACHINE" ) )
        root_key = HKEY_LOCAL_MACHINE;
    else if( !strcmp( root, "HKEY_USERS" ) )
        root_key = HKEY_USERS;
    else if( !strcmp( root, "HKEY_PERFORMANCE_DATA" ) )
        root_key = HKEY_PERFORMANCE_DATA;
    else if( !strcmp( root, "HKEY_CURRENT_CONFIG" ) )
        root_key = HKEY_CURRENT_CONFIG;
    else
        return NULL;
    
    if( RegOpenKeyEx( root_key, dir, 0, KEY_READ, &key_handle ) )
        return NULL; /* no need for a RegClose, so return direct */
    
    nbytes = 1;
    if( RegQueryValueEx( key_handle, name, 0, NULL, NULL, &nbytes ) )
        goto leave;
    result = malloc( (n1=nbytes+1) );
    if( !result )
        goto leave;
    if( RegQueryValueEx( key_handle, name, 0, NULL, result, &n1 ) ) {
        safe_free(result); result = NULL;
        goto leave;
    }
    result[nbytes] = 0; /* make sure it is really a string  */
    
leave:
    RegCloseKey( key_handle );
    return result;
}


const char *
_gpgme_get_gpg_path (int cleanup)
{
    static char *gpg_program = NULL;
    
    if (cleanup) {
	if (gpg_program) {
	    free (gpg_program);
	    gpg_program = NULL;
	}
	return NULL;
    }

    if (_gpgme_gpg_program)
        return _gpgme_gpg_program;

    if (!gpg_program) {
        gpg_program = read_w32_registry_string (NULL,
                                                "Software\\GNU\\GnuPG", "gpgProgram");
        if (gpg_program) {
            int i;
            
            DEBUG1 ("found gpgProgram in registry: `%s'\n", gpg_program );
            for (i=0; gpg_program[i]; i++) {
                if (gpg_program[i] == '/')
                    gpg_program[i] = '\\';
            }
        }
        else
            gpg_program = strdup ("C:\\GNUPG\\GPG.EXE");
    }
    
    return gpg_program;
}


const char *
_gpgme_get_gpg_optfile (int cleanup)
{
    static char *gpg_optfile = NULL;
    
    if (cleanup) {
	if (gpg_optfile) {
	    free (gpg_optfile);
	    gpg_optfile = NULL;
	}
	return NULL;
    }

    if (_gpgme_gpg_optfile)
        return(_gpgme_gpg_optfile);

    if (!gpg_optfile) {
	FILE *fp;
	gpg_optfile = read_w32_registry_string(
			    NULL, "Software\\GNU\\GnuPG", "OptFile" );
	if (gpg_optfile) {
	    int i;
	    for (i = 0; gpg_optfile[i]; i++) {
		if (gpg_optfile[i] == '/')
		    gpg_optfile[i] = '\\';
	    }
	    if (!strcmp(gpg_optfile, ""))
		return NULL; /* Then ignore the file */
	    fp = fopen (gpg_optfile, "r");
	    if (fp == NULL)
		return NULL;
	    fclose (fp);
	}
    }

    return gpg_optfile;
} /* _gpgme_get_gpg_optfile */


const char *
_gpgme_get_pubring (void)
{
    return _gpgme_gpg_pubring;
}


const char *
_gpgme_get_secring (void)
{
    return _gpgme_gpg_secring;
}


void
util_cleanup (void)
{
    _gpgme_get_gpg_optfile (1);
    _gpgme_get_gpg_path (1);
}

==> GPGee-1.3.1-source.zip: gpgee/gpgme/w32-util.c <==
/* w32-util.c - Utility functions for the W32 API
 *      Copyright (C) 1999 Free Software Foundation, Inc
 *      Copyright (C) 2001 Werner Koch (dd9jn)
 *      Copyright (C) 2002, 2005 Timo Schulz
 *
 * This file is part of MyGPGME.
 *
 * MyGPGME is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * MyGPGME is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>
#include <errno.h>
#include <sys/types.h>
#include <windows.h>
#include <process.h>
#include <io.h>

#include "util.h"

extern char *_gpgme_gpg_program;
extern char *_gpgme_gpg_optfile;
extern char *_gpgme_gpg_pubring;
extern char *_gpgme_gpg_secring;


static char *
expand_path (const char *path)
{
    DWORD len;
    char *p;

    len = ExpandEnvironmentStrings (path, NULL, 0);
    if (!len)
        return NULL;
    len += 1;
    p = calloc (1, len+1);
    if (!p)
        return NULL;
    len = ExpandEnvironmentStrings (path, p, len);
    if (!len) {
        free (p);
        return NULL;
    }
    return p;
}

/****************
 * Return a string from the Win32 Registry or NULL in case of
 * error.  Caller must release the return value.   A NULL for root
 * is an alias fro HKEY_CURRENT_USER
 */
static char *
read_w32_registry_string ( const char *root,
                           const char *dir, const char *name )
{
    HKEY root_key, key_handle;
    DWORD n1, nbytes;
    DWORD type;
    char *result = NULL;

    if (!root)
        root_key = HKEY_CURRENT_USER;
    else if (!strcmp(root, "HKEY_CLASSES_ROOT"))
        root_key = HKEY_CLASSES_ROOT;
    else if (!strcmp(root, "HKEY_CURRENT_USER"))
        root_key = HKEY_CURRENT_USER;
    else if (!strcmp(root, "HKEY_LOCAL_MACHINE"))
        root_key = HKEY_LOCAL_MACHINE;
    else if (!strcmp(root, "HKEY_USERS"))
        root_key = HKEY_USERS;
    else if (!strcmp(root, "HKEY_PERFORMANCE_DATA"))
        root_key = HKEY_PERFORMANCE_DATA;
    else if (!strcmp(root, "HKEY_CURRENT_CONFIG"))
        root_key = HKEY_CURRENT_CONFIG;
    else
        return NULL;

    if (RegOpenKeyEx(root_key, dir, 0, KEY_READ, &key_handle))
        return NULL; /* no need for a RegClose, so return direct */

    nbytes = 1;
    if( RegQueryValueEx( key_handle, name, 0, NULL, NULL, &nbytes ) )
        goto leave;
    result = calloc(1, (n1=nbytes+1) );
    if( !result )
        goto leave;
    if( RegQueryValueEx( key_handle, name, 0, &type, result, &n1 ) ) {
        safe_free(result); result = NULL;
        goto leave;
    }
    if (type == REG_EXPAND_SZ && strchr (result, '%')) {
       char *p = expand_path (result);
       DEBUG2 ("gpgProgram is expandable: %s -> %s\n", result, p);
       free (result);
       result = p;
    }
    result[nbytes] = 0; /* make sure it is really a string  */

leave:
    RegCloseKey( key_handle );
    return result;
}


const char *
_gpgme_get_gpg_path (int cleanup)
{
    static char *gpg_program = NULL;

    if (cleanup) {
        if (gpg_program) {
            free (gpg_program);
            gpg_program = NULL;
        }
        return NULL;
    }

    if (_gpgme_gpg_program)
        return _gpgme_gpg_program;

    if (!gpg_program) {
        gpg_program = read_w32_registry_string (NULL,
                                                "Software\\GNU\\GnuPG", "gpgProgram");
        if (gpg_program) {
            int i;

            DEBUG1 ("found gpgProgram in registry: `%s'\n", gpg_program );
            for (i=0; gpg_program[i]; i++) {
                if (gpg_program[i] == '/')
                    gpg_program[i] = '\\';
            }
        }
        else
            gpg_program = strdup ("C:\\GNUPG\\GPG.EXE");
    }

    return gpg_program;
}


const char *
_gpgme_get_gpg_optfile (int cleanup)
{
    static char *gpg_optfile = NULL;

    if (cleanup) {
        if (gpg_optfile) {
            free (gpg_optfile);
            gpg_optfile = NULL;
        }
        return NULL;
    }

    if (_gpgme_gpg_optfile)
        return(_gpgme_gpg_optfile);

    if (!gpg_optfile) {
        FILE *fp;
        gpg_optfile = read_w32_registry_string(
                            NULL, "Software\\GNU\\GnuPG", "OptFile" );
        if (gpg_optfile) {
            int i;
            for (i = 0; gpg_optfile[i]; i++) {
                if (gpg_optfile[i] == '/')
                    gpg_optfile[i] = '\\';
            }
            if (!strcmp(gpg_optfile, ""))
                return NULL; /* Then ignore the file */
            fp = fopen (gpg_optfile, "r");
            if (fp == NULL)
                return NULL;
            fclose (fp);
        }
    }

    return gpg_optfile;
} /* _gpgme_get_gpg_optfile */


const char *
_gpgme_get_pubring (void)
{
    return _gpgme_gpg_pubring;
}


const char *
_gpgme_get_secring (void)
{
    return _gpgme_gpg_secring;
}


void
util_cleanup (void)
{
    _gpgme_get_gpg_optfile (1);
    _gpgme_get_gpg_path (1);
}
